from scipy.constants import physical_constants, angstrom

angstrom_to_bohr = physical_constants['atomic unit of length'][0] / angstrom
ev_to_hartree = physical_constants['electron volt-hartree relationship'][0]
